@@ -95,6 +95,8 @@ struct _HippoCanvasHelper {
     guint tooltip_timeout_id;
     int last_window_x;
     int last_window_y;
+
+    int last_allocated_border;
     
     GSList *widget_items;
 
@@ -427,10 +429,23 @@ hippo_canvas_helper_size_allocate(HippoCanvasHelper *helper,
     /* g_debug("gtk allocate on canvas root %p canvas %p", helper->root, canvas); */
 
     if (helper->root != NULL) {
+        int border_width = GTK_CONTAINER(helper->widget)->border_width;
+        int child_width = allocation->width - border_width * 2;
+        int child_height = allocation->height - border_width * 2;
+        int old_width, old_height;
+        gboolean border_changed;
+        
+        hippo_canvas_item_get_allocation(helper->root, &old_width, &old_height);
+
+        border_changed = border_width = helper->last_allocated_border;
+        helper->last_allocated_border = border_width;
+
+        if (child_width != old_width || child_height != old_height || border_changed)
+            gtk_widget_queue_draw(helper->widget);
+        
         hippo_canvas_item_allocate(helper->root,
-                                   allocation->width - GTK_CONTAINER(helper->widget)->border_width * 2,
-                                   allocation->height  - GTK_CONTAINER(helper->widget)->border_width * 2,
-                                   FALSE);
+                                   child_width, child_height,
+                                   border_changed);
 
         /* Tooltip might be in the wrong place now */
         update_tooltip(helper, FALSE);
@@ -1060,7 +1075,7 @@ canvas_root_request_changed(HippoCanvasItem   *root,
     /* g_debug("queuing resize on canvas root %p canvas %p canvas container %p",
        root, canvas, helper->widget->parent); */
     if (!helper->fixing_up_resize_state)
-        gtk_widget_queue_resize(helper->widget);
+        gtk_widget_queue_resize_no_redraw(helper->widget);
 }
 
 static void
@@ -1193,7 +1208,7 @@ hippo_canvas_helper_set_width(HippoCanvasHelper *helper,
 
     helper->width = width;
 
-    gtk_widget_queue_resize(helper->widget);
+    gtk_widget_queue_resize_no_redraw(helper->widget);
 }
 
 /*
